
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef ivmob_MSG_LIB_H
#define ivmob_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "ivmobType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  IVMOB_MBOX_NAME                  genomInstanceName("ivmob")

#define  IVMOB_MAX_RQST_SIZE              128

#define  IVMOB_MBOX_RQST_SIZE       \
 (BUF_SIZE(IVMOB_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  IVMOB_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  IVMOB_MAX_REPLY_SIZE             52

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  IVMOB_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(IVMOB_MAX_REPLY_SIZE) \
   + BUF_SIZE(IVMOB_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define IVMOB_INIT_RQST 0
#define IVMOB_ACTIVATE_RQST 1
#define IVMOB_SETCMDPARAM_RQST 2
#define IVMOB_SETKINEMATICSPARAM_RQST 3
#define IVMOB_GETKINEMATICSPARAM_RQST 4
#define IVMOB_GETDYNAMICSPARAM_RQST 5
#define IVMOB_SETDYNAMICSPARAM_RQST 6
#define IVMOB_SETVARPARAM_RQST 7
#define IVMOB_STOP_RQST 8
#define IVMOB_TRACK_RQST 9
#define IVMOB_SETPOM_RQST 10
#define IVMOB_SETPOS_RQST 11
#define IVMOB_GYRO_RQST 12
#define IVMOB_GETGYRO_RQST 13
#define IVMOB_SETODOMETRYMETHOD_RQST 14
#define IVMOB_LOG_RQST 15
#define IVMOB_STOPLOG_RQST 16
#define IVMOB_LOGINTERNALDATA_RQST 17
#define IVMOB_STOPINTERNALDATALOG_RQST 18
#define IVMOB_GOTO_RQST 19
#define IVMOB_ABORT_RQST 20


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS ivmobClientInit (CLIENT_ID *pClientId);

STATUS ivmobClientEnd (CLIENT_ID clientId);

int ivmobAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int ivmobAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int ivmobAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS ivmobInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING32 *(in_device),
					 int replyTimeOut);

extern int ivmobInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING32 *(in_device), 
					
					int *activity, int *bilan);

extern int ivmobInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING32 *(in_device), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobActivateRqstSend (CLIENT_ID clientId, int *pRqstId,
					 int *(in_active),
					 int replyTimeOut);

extern int ivmobActivateReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobActivateRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					int *(in_active), 
					
					int *activity, int *bilan);

extern int ivmobActivateRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					int *(in_active), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobSetCmdParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_CMD_PARAM *(in_cmd),
					 int replyTimeOut);

int ivmobSetCmdParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobSetCmdParamRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_CMD_PARAM *(in_cmd), 
				 
				 int *bilan);




extern STATUS ivmobSetKinematicsParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IVMOB_KINEMATICS_STR *(in_kinematics),
					 int replyTimeOut);

int ivmobSetKinematicsParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobSetKinematicsParamRqstAndRcv (CLIENT_ID clientId, 
				 IVMOB_KINEMATICS_STR *(in_kinematics), 
				 
				 int *bilan);




extern STATUS ivmobGetKinematicsParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobGetKinematicsParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IVMOB_KINEMATICS_STR *(out_kinematics),
			       int *bilan);

int ivmobGetKinematicsParamRqstAndRcv (CLIENT_ID clientId, 
				  
				 IVMOB_KINEMATICS_STR *(out_kinematics),
				 int *bilan);




extern STATUS ivmobGetDynamicsParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobGetDynamicsParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IVMOB_DYNAMIC_STR *(out_dynamics),
			       int *bilan);

int ivmobGetDynamicsParamRqstAndRcv (CLIENT_ID clientId, 
				  
				 IVMOB_DYNAMIC_STR *(out_dynamics),
				 int *bilan);




extern STATUS ivmobSetDynamicsParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IVMOB_DYNAMIC_STR *(in_dynamics),
					 int replyTimeOut);

int ivmobSetDynamicsParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobSetDynamicsParamRqstAndRcv (CLIENT_ID clientId, 
				 IVMOB_DYNAMIC_STR *(in_dynamics), 
				 
				 int *bilan);




extern STATUS ivmobSetVarParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IVMOB_VAR_PARAMS *(in_varParams),
					 int replyTimeOut);

int ivmobSetVarParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobSetVarParamRqstAndRcv (CLIENT_ID clientId, 
				 IVMOB_VAR_PARAMS *(in_varParams), 
				 
				 int *bilan);




extern STATUS ivmobStopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobStopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobStopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS ivmobTrackRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IVMOB_TRACK_PARAMS *(in_trackParams),
					 int replyTimeOut);

extern int ivmobTrackReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobTrackRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IVMOB_TRACK_PARAMS *(in_trackParams), 
					
					int *activity, int *bilan);

extern int ivmobTrackRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IVMOB_TRACK_PARAMS *(in_trackParams), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobSetPomRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GENPOS_POSTER_NAME *(in_mePosterName),
					 int replyTimeOut);

extern int ivmobSetPomReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobSetPomRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity, int *bilan);

extern int ivmobSetPomRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobSetPosRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IVMOB_CART_3DCONFIG *(in_setPosStr),
					 int replyTimeOut);

int ivmobSetPosReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobSetPosRqstAndRcv (CLIENT_ID clientId, 
				 IVMOB_CART_3DCONFIG *(in_setPosStr), 
				 
				 int *bilan);




extern STATUS ivmobGyroRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IVMOB_GYRO_PARAMS *(in_params),
					 int replyTimeOut);

extern int ivmobGyroReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobGyroRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IVMOB_GYRO_PARAMS *(in_params), 
					
					int *activity, int *bilan);

extern int ivmobGyroRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IVMOB_GYRO_PARAMS *(in_params), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobGetGyroRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobGetGyroReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IVMOB_GYRO_PARAMS *(out_params),
			       int *bilan);

int ivmobGetGyroRqstAndRcv (CLIENT_ID clientId, 
				  
				 IVMOB_GYRO_PARAMS *(out_params),
				 int *bilan);




extern STATUS ivmobSetOdometryMethodRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IVMOB_ODOMETRY_METHOD *(in_odometryCommand),
					 int replyTimeOut);

extern int ivmobSetOdometryMethodReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobSetOdometryMethodRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IVMOB_ODOMETRY_METHOD *(in_odometryCommand), 
					
					int *activity, int *bilan);

extern int ivmobSetOdometryMethodRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IVMOB_ODOMETRY_METHOD *(in_odometryCommand), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobLogRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING128 *(in_fileName),
					 int replyTimeOut);

extern int ivmobLogReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobLogRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity, int *bilan);

extern int ivmobLogRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobStopLogRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobStopLogReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobStopLogRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS ivmobLogInternalDataRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING128 *(in_fileName),
					 int replyTimeOut);

extern int ivmobLogInternalDataReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int ivmobLogInternalDataRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity, int *bilan);

extern int ivmobLogInternalDataRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING128 *(in_fileName), 
					
					int *activity,
					int *bilan);




extern STATUS ivmobStopInternalDataLogRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int ivmobStopInternalDataLogReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobStopInternalDataLogRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS ivmobGotoRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_REF *(in_ref),
					 int replyTimeOut);

int ivmobGotoReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int ivmobGotoRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_REF *(in_ref), 
				 
				 int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* ivmob_MSG_LIB_H */

